@@ -1,18 +1,41 @@
 import os
 import re
-import urllib
 
-from django.conf import settings
-from django.contrib.auth import SESSION_KEY, REDIRECT_FIELD_NAME
-from django.contrib.auth.forms import AuthenticationForm
+from django.conf import global_settings, settings
 from django.contrib.sites.models import Site, RequestSite
 from django.contrib.auth.models import User
-from django.test import TestCase
 from django.core import mail
 from django.core.exceptions import SuspiciousOperation
-from django.core.urlresolvers import reverse
-from django.http import QueryDict
+from django.core.urlresolvers import reverse, NoReverseMatch
+from django.http import QueryDict, HttpRequest
+from django.utils.encoding import force_text
+from django.utils.html import escape
+from django.utils.http import urlquote
+from django.utils._os import upath
+from django.test import TestCase
+from django.test.utils import override_settings
+from django.middleware.csrf import CsrfViewMiddleware
+from django.contrib.sessions.middleware import SessionMiddleware
 
+from django.contrib.auth import SESSION_KEY, REDIRECT_FIELD_NAME
+from django.contrib.auth.forms import (AuthenticationForm, PasswordChangeForm,
+                SetPasswordForm, PasswordResetForm)
+from django.contrib.auth.tests.utils import skipIfCustomUser
+from django.contrib.auth.views import login as login_view
+
+
+@override_settings(
+    LANGUAGES=(
+        ('en', 'English'),
+    ),
+    LANGUAGE_CODE='en',
+    TEMPLATE_LOADERS=global_settings.TEMPLATE_LOADERS,
+    TEMPLATE_DIRS=(
+        os.path.join(os.path.dirname(upath(__file__)), 'templates'),
+    ),
+    USE_TZ=False,
+    PASSWORD_HASHERS=('django.contrib.auth.hashers.SHA1PasswordHasher',),
+)
 class AuthViewsTestCase(TestCase):
     """
     Helper base class for all the follow test cases.
@@ -20,31 +43,46 @@ class AuthViewsTestCase(TestCase):
     fixtures = ['authtestdata.json']
     urls = 'django.contrib.auth.tests.urls'
 
-    def setUp(self):
-        self.old_LANGUAGES = settings.LANGUAGES
-        self.old_LANGUAGE_CODE = settings.LANGUAGE_CODE
-        settings.LANGUAGES = (('en', 'English'),)
-        settings.LANGUAGE_CODE = 'en'
-        self.old_TEMPLATE_DIRS = settings.TEMPLATE_DIRS
-        settings.TEMPLATE_DIRS = (
-            os.path.join(os.path.dirname(__file__), 'templates'),
-        )
-
-    def tearDown(self):
-        settings.LANGUAGES = self.old_LANGUAGES
-        settings.LANGUAGE_CODE = self.old_LANGUAGE_CODE
-        settings.TEMPLATE_DIRS = self.old_TEMPLATE_DIRS
-
     def login(self, password='password'):
         response = self.client.post('/login/', {
             'username': 'testclient',
-            'password': password
-            }
-        )
+            'password': password,
+            })
         self.assertEqual(response.status_code, 302)
         self.assertTrue(response['Location'].endswith(settings.LOGIN_REDIRECT_URL))
         self.assertTrue(SESSION_KEY in self.client.session)
 
+    def assertContainsEscaped(self, response, text, **kwargs):
+        return self.assertContains(response, escape(force_text(text)), **kwargs)
+
+
+@skipIfCustomUser
+class AuthViewNamedURLTests(AuthViewsTestCase):
+    urls = 'django.contrib.auth.urls'
+
+    def test_named_urls(self):
+        "Named URLs should be reversible"
+        expected_named_urls = [
+            ('login', [], {}),
+            ('logout', [], {}),
+            ('password_change', [], {}),
+            ('password_change_done', [], {}),
+            ('password_reset', [], {}),
+            ('password_reset_done', [], {}),
+            ('password_reset_confirm', [], {
+                'uidb36': 'aaaaaaa',
+                'token': '1111-aaaaa',
+            }),
+            ('password_reset_complete', [], {}),
+        ]
+        for name, args, kwargs in expected_named_urls:
+            try:
+                reverse(name, args=args, kwargs=kwargs)
+            except NoReverseMatch:
+                self.fail("Reversal of url named '%s' failed with NoReverseMatch" % name)
+
+
+@skipIfCustomUser
 class PasswordResetTest(AuthViewsTestCase):
 
     def test_email_not_found(self):
@@ -52,7 +90,7 @@ class PasswordResetTest(AuthViewsTestCase):
         response = self.client.get('/password_reset/')
         self.assertEqual(response.status_code, 200)
         response = self.client.post('/password_reset/', {'email': 'not_a_real_email@email.com'})
-        self.assertContains(response, "That e-mail address doesn&#39;t have an associated user account")
+        self.assertContainsEscaped(response, PasswordResetForm.error_messages['unknown'])
         self.assertEqual(len(mail.outbox), 0)
 
     def test_email_found(self):
@@ -70,6 +108,7 @@ class PasswordResetTest(AuthViewsTestCase):
         self.assertEqual(len(mail.outbox), 1)
         self.assertEqual("staffmember@example.com", mail.outbox[0].from_email)
 
+    @override_settings(ALLOWED_HOSTS=['adminsite.com'])
     def test_admin_reset(self):
         "If the reset view is marked as being for admin, the HTTP_HOST header is used for a domain override."
         response = self.client.post('/admin_password_reset/',
@@ -81,6 +120,8 @@ class PasswordResetTest(AuthViewsTestCase):
         self.assertTrue("http://adminsite.com" in mail.outbox[0].body)
         self.assertEqual(settings.DEFAULT_FROM_EMAIL, mail.outbox[0].from_email)
 
+    # Skip any 500 handler action (like sending more mail...)
+    @override_settings(DEBUG_PROPAGATE_EXCEPTIONS=True)
     def test_poisoned_http_host(self):
         "Poisoned HTTP_HOST headers can't be used for reset emails"
         # This attack is based on the way browsers handle URLs. The colon
@@ -90,22 +131,22 @@ class PasswordResetTest(AuthViewsTestCase):
         # produce a meaningful reset URL, we need to be certain that the
         # HTTP_HOST header isn't poisoned. This is done as a check when get_host()
         # is invoked, but we check here as a practical consequence.
-        def test_host_poisoning():
+        with self.assertRaises(SuspiciousOperation):
             self.client.post('/password_reset/',
                 {'email': 'staffmember@example.com'},
                 HTTP_HOST='www.example:dr.frankenstein@evil.tld'
             )
-        self.assertRaises(SuspiciousOperation, test_host_poisoning)
         self.assertEqual(len(mail.outbox), 0)
 
+    # Skip any 500 handler action (like sending more mail...)
+    @override_settings(DEBUG_PROPAGATE_EXCEPTIONS=True)
     def test_poisoned_http_host_admin_site(self):
         "Poisoned HTTP_HOST headers can't be used for reset emails on admin views"
-        def test_host_poisoning():
+        with self.assertRaises(SuspiciousOperation):
             self.client.post('/admin_password_reset/',
                 {'email': 'staffmember@example.com'},
                 HTTP_HOST='www.example:dr.frankenstein@evil.tld'
             )
-        self.assertRaises(SuspiciousOperation, test_host_poisoning)
         self.assertEqual(len(mail.outbox), 0)
 
     def _test_confirm_start(self):
@@ -124,39 +165,37 @@ class PasswordResetTest(AuthViewsTestCase):
         url, path = self._test_confirm_start()
         response = self.client.get(path)
         # redirect to a 'complete' page:
-        self.assertEqual(response.status_code, 200)
-        self.assertTrue("Please enter your new password" in response.content)
+        self.assertContains(response, "Please enter your new password")
 
     def test_confirm_invalid(self):
         url, path = self._test_confirm_start()
         # Let's munge the token in the path, but keep the same length,
         # in case the URLconf will reject a different length.
-        path = path[:-5] + ("0"*4) + path[-1]
+        path = path[:-5] + ("0" * 4) + path[-1]
 
         response = self.client.get(path)
-        self.assertEqual(response.status_code, 200)
-        self.assertTrue("The password reset link was invalid" in response.content)
+        self.assertContains(response, "The password reset link was invalid")
 
     def test_confirm_invalid_user(self):
         # Ensure that we get a 200 response for a non-existant user, not a 404
         response = self.client.get('/reset/123456-1-1/')
-        self.assertEqual(response.status_code, 200)
-        self.assertTrue("The password reset link was invalid" in response.content)
+        self.assertContains(response, "The password reset link was invalid")
 
     def test_confirm_overflow_user(self):
         # Ensure that we get a 200 response for a base36 user id that overflows int
         response = self.client.get('/reset/zzzzzzzzzzzzz-1-1/')
-        self.assertEqual(response.status_code, 200)
-        self.assertTrue("The password reset link was invalid" in response.content)
+        self.assertContains(response, "The password reset link was invalid")
 
     def test_confirm_invalid_post(self):
         # Same as test_confirm_invalid, but trying
         # to do a POST instead.
         url, path = self._test_confirm_start()
-        path = path[:-5] + ("0"*4) + path[-1]
+        path = path[:-5] + ("0" * 4) + path[-1]
 
-        response = self.client.post(path, {'new_password1': 'anewpassword',
-                                           'new_password2':' anewpassword'})
+        self.client.post(path, {
+            'new_password1': 'anewpassword',
+            'new_password2': ' anewpassword',
+        })
         # Check the password has not been changed
         u = User.objects.get(email='staffmember@example.com')
         self.assertTrue(not u.check_password("anewpassword"))
@@ -173,26 +212,49 @@ class PasswordResetTest(AuthViewsTestCase):
 
         # Check we can't use the link again
         response = self.client.get(path)
-        self.assertEqual(response.status_code, 200)
-        self.assertTrue("The password reset link was invalid" in response.content)
+        self.assertContains(response, "The password reset link was invalid")
 
     def test_confirm_different_passwords(self):
         url, path = self._test_confirm_start()
         response = self.client.post(path, {'new_password1': 'anewpassword',
-                                           'new_password2':' x'})
-        self.assertEqual(response.status_code, 200)
-        self.assertTrue("The two password fields didn&#39;t match" in response.content)
+                                           'new_password2': 'x'})
+        self.assertContainsEscaped(response, SetPasswordForm.error_messages['password_mismatch'])
 
+
+@override_settings(AUTH_USER_MODEL='auth.CustomUser')
+class CustomUserPasswordResetTest(AuthViewsTestCase):
+    fixtures = ['custom_user.json']
+
+    def _test_confirm_start(self):
+        # Start by creating the email
+        response = self.client.post('/password_reset/', {'email': 'staffmember@example.com'})
+        self.assertEqual(response.status_code, 302)
+        self.assertEqual(len(mail.outbox), 1)
+        return self._read_signup_email(mail.outbox[0])
+
+    def _read_signup_email(self, email):
+        urlmatch = re.search(r"https?://[^/]*(/.*reset/\S*)", email.body)
+        self.assertTrue(urlmatch is not None, "No URL found in sent email")
+        return urlmatch.group(), urlmatch.groups()[0]
+
+    def test_confirm_valid_custom_user(self):
+        url, path = self._test_confirm_start()
+        response = self.client.get(path)
+        # redirect to a 'complete' page:
+        self.assertContains(response, "Please enter your new password")
+
+
+@skipIfCustomUser
 class ChangePasswordTest(AuthViewsTestCase):
 
     def fail_login(self, password='password'):
         response = self.client.post('/login/', {
             'username': 'testclient',
-            'password': password
-            }
-        )
-        self.assertEqual(response.status_code, 200)
-        self.assertTrue("Please enter a correct username and password. Note that both fields are case-sensitive." in response.content)
+            'password': password,
+        })
+        self.assertContainsEscaped(response, AuthenticationForm.error_messages['invalid_login'] % {
+                'username': User._meta.get_field('username').verbose_name
+            })
 
     def logout(self):
         response = self.client.get('/logout/')
@@ -203,10 +265,8 @@ class ChangePasswordTest(AuthViewsTestCase):
             'old_password': 'donuts',
             'new_password1': 'password1',
             'new_password2': 'password1',
-            }
-        )
-        self.assertEqual(response.status_code, 200)
-        self.assertTrue("Your old password was entered incorrectly. Please enter it again." in response.content)
+        })
+        self.assertContainsEscaped(response, PasswordChangeForm.error_messages['password_incorrect'])
 
     def test_password_change_fails_with_mismatched_passwords(self):
         self.login()
@@ -214,10 +274,8 @@ class ChangePasswordTest(AuthViewsTestCase):
             'old_password': 'password',
             'new_password1': 'password1',
             'new_password2': 'donuts',
-            }
-        )
-        self.assertEqual(response.status_code, 200)
-        self.assertTrue("The two password fields didn&#39;t match." in response.content)
+        })
+        self.assertContainsEscaped(response, SetPasswordForm.error_messages['password_mismatch'])
 
     def test_password_change_succeeds(self):
         self.login()
@@ -225,13 +283,30 @@ class ChangePasswordTest(AuthViewsTestCase):
             'old_password': 'password',
             'new_password1': 'password1',
             'new_password2': 'password1',
-            }
-        )
+        })
         self.assertEqual(response.status_code, 302)
         self.assertTrue(response['Location'].endswith('/password_change/done/'))
         self.fail_login()
         self.login(password='password1')
 
+    def test_password_change_done_succeeds(self):
+        self.login()
+        response = self.client.post('/password_change/', {
+            'old_password': 'password',
+            'new_password1': 'password1',
+            'new_password2': 'password1',
+        })
+        self.assertEqual(response.status_code, 302)
+        self.assertTrue(response['Location'].endswith('/password_change/done/'))
+
+    def test_password_change_done_fails(self):
+        with self.settings(LOGIN_URL='/login/'):
+            response = self.client.get('/password_change/done/')
+            self.assertEqual(response.status_code, 302)
+            self.assertTrue(response['Location'].endswith('/login/?next=/password_change/done/'))
+
+
+@skipIfCustomUser
 class LoginTest(AuthViewsTestCase):
 
     def test_current_site_in_context_after_login(self):
@@ -251,20 +326,22 @@ class LoginTest(AuthViewsTestCase):
 
         # Those URLs should not pass the security check
         for bad_url in ('http://example.com',
+                        'http:///example.com',
                         'https://example.com',
                         'ftp://exampel.com',
-                        '//example.com'):
+                        '///example.com',
+                        '//example.com',
+                        'javascript:alert("XSS")'):
 
             nasty_url = '%(url)s?%(next)s=%(bad_url)s' % {
                 'url': login_url,
                 'next': REDIRECT_FIELD_NAME,
-                'bad_url': urllib.quote(bad_url)
+                'bad_url': urlquote(bad_url),
             }
             response = self.client.post(nasty_url, {
                 'username': 'testclient',
                 'password': password,
-                }
-            )
+            })
             self.assertEqual(response.status_code, 302)
             self.assertFalse(bad_url in response['Location'],
                              "%s should be blocked" % bad_url)
@@ -274,27 +351,60 @@ class LoginTest(AuthViewsTestCase):
                          '/view/?param=https://example.com',
                          '/view?param=ftp://exampel.com',
                          'view/?param=//example.com',
-                         'https:///',
+                         'https://testserver/',
+                         'HTTPS://testserver/',
                          '//testserver/',
-                         '/url%20with%20spaces/', # see ticket #12534
-                         ):
+                         '/url%20with%20spaces/'):  # see ticket #12534
             safe_url = '%(url)s?%(next)s=%(good_url)s' % {
                 'url': login_url,
                 'next': REDIRECT_FIELD_NAME,
-                'good_url': urllib.quote(good_url)
+                'good_url': urlquote(good_url),
             }
             response = self.client.post(safe_url, {
                     'username': 'testclient',
                     'password': password,
-                }
-            )
+            })
             self.assertEqual(response.status_code, 302)
             self.assertTrue(good_url in response['Location'],
                             "%s should be allowed" % good_url)
 
+    def test_login_csrf_rotate(self, password='password'):
+        """
+        Makes sure that a login rotates the currently-used CSRF token.
+        """
+        # Do a GET to establish a CSRF token
+        # TestClient isn't used here as we're testing middleware, essentially.
+        req = HttpRequest()
+        CsrfViewMiddleware().process_view(req, login_view, (), {})
+        req.META["CSRF_COOKIE_USED"] = True
+        resp = login_view(req)
+        resp2 = CsrfViewMiddleware().process_response(req, resp)
+        csrf_cookie = resp2.cookies.get(settings.CSRF_COOKIE_NAME, None)
+        token1 = csrf_cookie.coded_value
 
+        # Prepare the POST request
+        req = HttpRequest()
+        req.COOKIES[settings.CSRF_COOKIE_NAME] = token1
+        req.method = "POST"
+        req.POST = {'username': 'testclient', 'password': password, 'csrfmiddlewaretoken': token1}
+        req.REQUEST = req.POST
+
+        # Use POST request to log in
+        SessionMiddleware().process_request(req)
+        CsrfViewMiddleware().process_view(req, login_view, (), {})
+        req.META["SERVER_NAME"] = "testserver"  # Required to have redirect work in login view
+        req.META["SERVER_PORT"] = 80
+        resp = login_view(req)
+        resp2 = CsrfViewMiddleware().process_response(req, resp)
+        csrf_cookie = resp2.cookies.get(settings.CSRF_COOKIE_NAME, None)
+        token2 = csrf_cookie.coded_value
+
+        # Check the CSRF token switched
+        self.assertNotEqual(token1, token2)
+
+
+@skipIfCustomUser
 class LoginURLSettings(AuthViewsTestCase):
-    urls = 'django.contrib.auth.tests.urls'
 
     def setUp(self):
         super(LoginURLSettings, self).setUp()
@@ -315,8 +425,8 @@ class LoginURLSettings(AuthViewsTestCase):
         login_required_url = self.get_login_required_url(login_url)
         querystring = QueryDict('', mutable=True)
         querystring['next'] = '/login_required/'
-        self.assertEqual(login_required_url,
-             'http://testserver%s?%s' % (login_url, querystring.urlencode('/')))
+        self.assertEqual(login_required_url, 'http://testserver%s?%s' %
+                         (login_url, querystring.urlencode('/')))
 
     def test_remote_login_url(self):
         login_url = 'http://remote.example.com/login'
@@ -352,8 +462,8 @@ class LoginURLSettings(AuthViewsTestCase):
                                                     querystring.urlencode('/')))
 
 
+@skipIfCustomUser
 class LogoutTest(AuthViewsTestCase):
-    urls = 'django.contrib.auth.tests.urls'
 
     def confirm_logged_out(self):
         self.assertTrue(SESSION_KEY not in self.client.session)
@@ -362,8 +472,7 @@ class LogoutTest(AuthViewsTestCase):
         "Logout without next_page option renders the default template"
         self.login()
         response = self.client.get('/logout/')
-        self.assertEqual(200, response.status_code)
-        self.assertTrue('Logged out' in response.content)
+        self.assertContains(response, 'Logged out')
         self.confirm_logged_out()
 
     def test_14377(self):
@@ -414,14 +523,16 @@ class LogoutTest(AuthViewsTestCase):
 
         # Those URLs should not pass the security check
         for bad_url in ('http://example.com',
+                        'http:///example.com',
                         'https://example.com',
                         'ftp://exampel.com',
-                        '//example.com'
-                        ):
+                        '///example.com',
+                        '//example.com',
+                        'javascript:alert("XSS")'):
             nasty_url = '%(url)s?%(next)s=%(bad_url)s' % {
                 'url': logout_url,
                 'next': REDIRECT_FIELD_NAME,
-                'bad_url': urllib.quote(bad_url)
+                'bad_url': urlquote(bad_url),
             }
             self.login()
             response = self.client.get(nasty_url)
@@ -435,14 +546,14 @@ class LogoutTest(AuthViewsTestCase):
                          '/view/?param=https://example.com',
                          '/view?param=ftp://exampel.com',
                          'view/?param=//example.com',
-                         'https:///',
+                         'https://testserver/',
+                         'HTTPS://testserver/',
                          '//testserver/',
-                         '/url%20with%20spaces/', # see ticket #12534
-                         ):
+                         '/url%20with%20spaces/'):  # see ticket #12534
             safe_url = '%(url)s?%(next)s=%(good_url)s' % {
                 'url': logout_url,
                 'next': REDIRECT_FIELD_NAME,
-                'good_url': urllib.quote(good_url)
+                'good_url': urlquote(good_url),
             }
             self.login()
             response = self.client.get(safe_url)
@@ -450,3 +561,18 @@ class LogoutTest(AuthViewsTestCase):
             self.assertTrue(good_url in response['Location'],
                             "%s should be allowed" % good_url)
             self.confirm_logged_out()
+
+@skipIfCustomUser
+class ChangelistTests(AuthViewsTestCase):
+    urls = 'django.contrib.auth.tests.urls_admin'
+
+    # #20078 - users shouldn't be allowed to guess password hashes via
+    # repeated password__startswith queries.
+    def test_changelist_disallows_password_lookups(self):
+        # Make me a superuser before loging in.
+        User.objects.filter(username='testclient').update(is_staff=True, is_superuser=True)
+        self.login()
+
+        # A lookup that tries to filter on password isn't OK
+        with self.assertRaises(SuspiciousOperation):
+            response = self.client.get('/admin/auth/user/?password__startswith=sha1$')
