@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals
 
 import tempfile
 import os
@@ -27,12 +27,15 @@ from .models import (Article, Chapter, Account, Media, Child, Parent, Picture,
     Album, Question, Answer, ComplexSortedPerson, PrePopulatedPostLargeSlug,
     AdminOrderedField, AdminOrderedModelMethod, AdminOrderedAdminMethod,
     AdminOrderedCallable, Report, Color2, UnorderedObject, MainPrepopulated,
-    RelatedPrepopulated, ReferencedByParent, ChildOfReferer, M2MReference,
-    ReferencedByInline, InlineReference, InlineReferer, Ingredient)
+    RelatedPrepopulated, UndeletableObject, UserMessenger, Simple, Choice,
+    ShortMessage, Telegram, ReferencedByParent, ChildOfReferer, M2MReference)
 
 
 def callable_year(dt_value):
-    return dt_value.year
+    try:
+        return dt_value.year
+    except AttributeError:
+        return None
 callable_year.admin_order_field = 'date'
 
 
@@ -172,12 +175,12 @@ class PersonAdmin(admin.ModelAdmin):
 
 class FooAccount(Account):
     """A service-specific account of type Foo."""
-    servicename = u'foo'
+    servicename = 'foo'
 
 
 class BarAccount(Account):
     """A service-specific account of type Bar."""
-    servicename = u'bar'
+    servicename = 'bar'
 
 
 class FooAccountAdmin(admin.StackedInline):
@@ -344,7 +347,10 @@ class LinkInline(admin.TabularInline):
     model = Link
     extra = 1
 
-    readonly_fields = ("posted",)
+    readonly_fields = ("posted", "multiline")
+
+    def multiline(self, instance):
+        return "InlineMultiline\ntest\nstring"
 
 
 class SubPostInline(admin.TabularInline):
@@ -386,7 +392,10 @@ class PrePopulatedPostAdmin(admin.ModelAdmin):
 
 class PostAdmin(admin.ModelAdmin):
     list_display = ['title', 'public']
-    readonly_fields = ('posted', 'awesomeness_level', 'coolness', 'value', lambda obj: "foo")
+    readonly_fields = (
+        'posted', 'awesomeness_level', 'coolness', 'value', 'multiline',
+        lambda obj: "foo"
+    )
 
     inlines = [
         LinkInline
@@ -400,6 +409,10 @@ class PostAdmin(admin.ModelAdmin):
 
     def value(self, instance):
         return 1000
+
+    def multiline(self, instance):
+        return "Multiline\ntest\nstring"
+
     value.short_description = 'Value in $US'
 
 
@@ -427,25 +440,54 @@ class FoodDeliveryAdmin(admin.ModelAdmin):
     list_editable = ('driver', 'restaurant')
 
 
+class CoverLetterAdmin(admin.ModelAdmin):
+    """
+    A ModelAdmin with a custom queryset() method that uses defer(), to test
+    verbose_name display in messages shown after adding/editing CoverLetter
+    instances.
+    Note that the CoverLetter model defines a __unicode__ method.
+    For testing fix for ticket #14529.
+    """
+
+    def queryset(self, request):
+        return super(CoverLetterAdmin, self).queryset(request).defer('date_written')
+
+
 class PaperAdmin(admin.ModelAdmin):
     """
     A ModelAdmin with a custom queryset() method that uses only(), to test
-    verbose_name display in messages shown after adding Paper instances.
+    verbose_name display in messages shown after adding/editing Paper
+    instances.
+    For testing fix for ticket #14529.
     """
 
     def queryset(self, request):
         return super(PaperAdmin, self).queryset(request).only('title')
 
 
-class CoverLetterAdmin(admin.ModelAdmin):
+class ShortMessageAdmin(admin.ModelAdmin):
     """
-    A ModelAdmin with a custom queryset() method that uses only(), to test
-    verbose_name display in messages shown after adding CoverLetter instances.
-    Note that the CoverLetter model defines a __unicode__ method.
+    A ModelAdmin with a custom queryset() method that uses defer(), to test
+    verbose_name display in messages shown after adding/editing ShortMessage
+    instances.
+    For testing fix for ticket #14529.
     """
 
     def queryset(self, request):
-        return super(CoverLetterAdmin, self).queryset(request).defer('date_written')
+        return super(ShortMessageAdmin, self).queryset(request).defer('timestamp')
+
+
+class TelegramAdmin(admin.ModelAdmin):
+    """
+    A ModelAdmin with a custom queryset() method that uses only(), to test
+    verbose_name display in messages shown after adding/editing Telegram
+    instances.
+    Note that the Telegram model defines a __unicode__ method.
+    For testing fix for ticket #14529.
+    """
+
+    def queryset(self, request):
+        return super(TelegramAdmin, self).queryset(request).only('title')
 
 
 class StoryForm(forms.ModelForm):
@@ -570,13 +612,46 @@ class UnorderedObjectAdmin(admin.ModelAdmin):
     list_per_page = 2
 
 
-
-class InlineReferenceInline(admin.TabularInline):
-    model = InlineReference
+class UndeletableObjectAdmin(admin.ModelAdmin):
+    def change_view(self, *args, **kwargs):
+        kwargs['extra_context'] = {'show_delete': False}
+        return super(UndeletableObjectAdmin, self).change_view(*args, **kwargs)
 
 
-class InlineRefererAdmin(admin.ModelAdmin):
-    inlines = [InlineReferenceInline]
+def callable_on_unknown(obj):
+    return obj.unknown
+
+
+class AttributeErrorRaisingAdmin(admin.ModelAdmin):
+    list_display = [callable_on_unknown, ]
+
+class MessageTestingAdmin(admin.ModelAdmin):
+    actions = ["message_debug", "message_info", "message_success",
+               "message_warning", "message_error", "message_extra_tags"]
+
+    def message_debug(self, request, selected):
+        self.message_user(request, "Test debug", level="debug")
+
+    def message_info(self, request, selected):
+        self.message_user(request, "Test info", level="info")
+
+    def message_success(self, request, selected):
+        self.message_user(request, "Test success", level="success")
+
+    def message_warning(self, request, selected):
+        self.message_user(request, "Test warning", level="warning")
+
+    def message_error(self, request, selected):
+        self.message_user(request, "Test error", level="error")
+
+    def message_extra_tags(self, request, selected):
+        self.message_user(request, "Test tags", extra_tags="extra_tag")
+
+
+class ChoiceList(admin.ModelAdmin):
+    list_display = ['choice']
+    readonly_fields = ['choice']
+    fields = ['choice']
 
 
 site = admin.AdminSite(name="admin")
@@ -620,16 +695,17 @@ site.register(FoodDelivery, FoodDeliveryAdmin)
 site.register(RowLevelChangePermissionModel, RowLevelChangePermissionModelAdmin)
 site.register(Paper, PaperAdmin)
 site.register(CoverLetter, CoverLetterAdmin)
+site.register(ShortMessage, ShortMessageAdmin)
+site.register(Telegram, TelegramAdmin)
 site.register(Story, StoryAdmin)
 site.register(OtherStory, OtherStoryAdmin)
 site.register(Report, ReportAdmin)
 site.register(MainPrepopulated, MainPrepopulatedAdmin)
 site.register(UnorderedObject, UnorderedObjectAdmin)
+site.register(UndeletableObject, UndeletableObjectAdmin)
 site.register(ReferencedByParent)
 site.register(ChildOfReferer)
 site.register(M2MReference)
-site.register(ReferencedByInline)
-site.register(InlineReferer, InlineRefererAdmin)
 
 # We intentionally register Promo and ChapterXtra1 but not Chapter nor ChapterXtra2.
 # That way we cover all four cases:
@@ -638,7 +714,7 @@ site.register(InlineReferer, InlineRefererAdmin)
 #     related OneToOne object registered in admin
 #     related OneToOne object not registered in admin
 # when deleting Book so as exercise all four troublesome (w.r.t escaping
-# and calling force_unicode to avoid problems on Python 2.3) paths through
+# and calling force_text to avoid problems on Python 2.3) paths through
 # contrib.admin.util's get_deleted_objects function.
 site.register(Book, inlines=[ChapterInline])
 site.register(Promo)
@@ -656,7 +732,9 @@ site.register(AdminOrderedModelMethod, AdminOrderedModelMethodAdmin)
 site.register(AdminOrderedAdminMethod, AdminOrderedAdminMethodAdmin)
 site.register(AdminOrderedCallable, AdminOrderedCallableAdmin)
 site.register(Color2, CustomTemplateFilterColorAdmin)
-site.register(Ingredient)
+site.register(Simple, AttributeErrorRaisingAdmin)
+site.register(UserMessenger, MessageTestingAdmin)
+site.register(Choice, ChoiceList)
 
 # Register core models we need in our tests
 from django.contrib.auth.models import User, Group
